"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./common");
function joinClassNames(classes) {
    if (classes.length === 1) {
        return classes[0];
    }
    if (classes.length === 2) {
        return classes.join(" and ");
    }
    classes[classes.length - 1] = `and ${classes[classes.length - 1]}`;
    return classes.join(", ");
}
exports.joinClassNames = joinClassNames;
exports.default = common_1.createRule({
    description: "Ensures that all of the class exported by a css file are used.",
    messages: {
        unusedClassName: "Class {{ classNames }} is exported but unused.",
        unusedClassNames: "Classes {{ classNames }} are exported but unused.",
    },
    create: context => {
        const cache = new common_1.Cache(context);
        const filenameToUnusedClasses = {};
        return {
            ImportDeclaration: (node) => {
                const result = cache.processImportDeclaration(node);
                if (result === null) {
                    return;
                }
                const { node: importNode, filename, classes, explicitImports } = result;
                const unusedClasses = filenameToUnusedClasses[filename] !== undefined
                    ? filenameToUnusedClasses[filename].unusedClasses
                    : new Set(classes);
                explicitImports.forEach(node => {
                    const className = node.imported.name;
                    unusedClasses.delete(className);
                });
                if (filenameToUnusedClasses[filename] === undefined) {
                    filenameToUnusedClasses[filename] = {
                        importNode,
                        unusedClasses,
                    };
                }
            },
            MemberExpression: (node) => {
                const result = cache.processMemberExpression(node);
                if (result === null) {
                    return;
                }
                const { filename, className } = result;
                if (filenameToUnusedClasses[filename]) {
                    filenameToUnusedClasses[filename].unusedClasses.delete(className);
                }
            },
            "Program:exit": () => {
                Object.keys(filenameToUnusedClasses).forEach(filename => {
                    const { importNode, unusedClasses } = filenameToUnusedClasses[filename];
                    if (unusedClasses.size > 0) {
                        const classNames = joinClassNames(Array.from(unusedClasses));
                        context.report({
                            messageId: unusedClasses.size === 1
                                ? "unusedClassName"
                                : "unusedClassNames",
                            node: importNode,
                            data: {
                                classNames,
                            },
                        });
                    }
                });
            },
        };
    },
});
